<?php
$OPTIONS['admin_setup'] = array('show' => 'admin_setup_show');

function admin_setup_show(&$args, $dom) {
    global $SETUP_CONSTANTS;
    auth_check(false, 'admin');
    title_set_l('Site preferences');

    $values = array();

    $contents = setup_read('site');
    if($contents !== false) {
	$values = unserialize($contents);
    }

    if(isset($args['first']) && $args['first'] === 'true') {
	theme_convert(THEME);
	page_stop_processing();
	message_info(l('You can reconfigure these items on Admin tools > Site preferences.'));
    }

    if(isset($args['apply']) && $args['apply'] === 'site') {
	$is_success = true;
	foreach($SETUP_CONSTANTS as $key => &$setup_constant) {
	    if(!empty($setup_constant['debug']) && DEBUG_MODE !== 'true')
		continue;
	    $name = 'const_' . $key;
	    $type = default_value($setup_constant['type'], 'string');

	    if($type === 'theme_file_image') {
		if(isset($args[$name . '_delete']) && $args[$name . '_delete'] === 'on') {
		    $values[$key] = '';
		    $delete_path = THEME_DIR_PATH . '/' . $setup_constant['path'];
		    if(isset($setup_constant['extensions'])) {
			foreach($setup_constant['extensions'] as $extension) {
			    if(is_file($delete_path . $extension))
				unlink($delete_path . $extension);
			}
		    } else if(is_file($delete_path)) {
			unlink($delete_path);
		    }
		    continue;
		}
		
		if(empty($_FILES[$name]) || !isset($_FILES[$name]['name']) || $_FILES[$name]['name'] === '')
		    continue;

		$path_info = pathinfo($_FILES[$name]['name']);
		$extension = strtolower($path_info['extension']);
		if(isset($setup_constant['extensions']) &&
		   !in_array($extension, $setup_constant['extensions'])) {
		    message_error(lf('You must enter a $term$', array('term' => l($setup_constant['description']))));
		    $is_success = false;
		    $setup_constant['error'] = true;
		    continue;
		}
		    
		$path = THEME_DIR_PATH . '/' . $setup_constant['path'] . $extension;
		if(!move_uploaded_file($_FILES[$name]['tmp_name'], $path)) {
		    message_error(l('Upload failed.'));
		    $is_success = false;
		    $setup_constant['error'] = true;
		    continue;
		}
		$values[$key] = THEME_URI . '/' . $setup_constant['path'] . $extension;
		if(isset($setup_constant['extensions'])) {
		    foreach($setup_constant['extensions'] as $extension) {
			$delete_path = THEME_DIR_PATH . '/' . $setup_constant['path'] . $extension;
			if($delete_path !== $path && is_file($delete_path)) {
			    unlink($delete_path);
			}
		    }
		}
		continue;
	    }

	    if(isset($args[$name])) {
		if($type == 'pagename')
		    $values[$key] = page_path_to_pagename($args[$name], '', false);
		//else if($type == 'color')
		//    $values[$key] = color_keyword_to_rgb($args[$name]);
		else
		    $values[$key] = $args[$name];
	    }

	    if(empty($setup_constant['allowempty']) && (!isset($values[$key]) || $values[$key] === '')) {
		message_error(lf('You must enter a $term$', array('term' => l($setup_constant['description']))));
		$is_success = false;
		$setup_constant['error'] = true;
		continue;
	    }

	    switch($type) {
	    case 'select':
		if(!isset($setup_constant['options'][$args[$name]]))
		    $setup_constant['error'] = true;
		break;
	    case 'timezone':
		$options = time_get_timezones();
		if(!isset($options[$args[$name]]))
		    $setup_constant['error'] = true;
		break;
	    case 'theme':
		$options = theme_get_themes();
		if(!isset($options[$args[$name]]))
		    $setup_constant['error'] = true;
		break;
	    case 'color':
		if($args[$name] !== '' && !color_is_valid($args[$name]))
		    $setup_constant['error'] = true;
		break;
	    }
	    if(!empty($setup_constant['error'])) {
		message_error(lf('Invalid $term$.', array('term' => l($setup_constant['description']))));
		$is_success = false;
	    }
	}

	if($is_success) {
	    $contents = serialize($values);
	    if(!setup_write('site', $contents))
		message_error(l('Could not save site preferences.'));
	    else {
		header('Location: ' . 
		       url_make('',
				array('option' => 'admin_setup',
				      'apply' => 'theme',
				      'first' => default_value($args['first'], ''))));
		exit(0);
	    }
	}
    } else if(isset($args['apply']) && $args['apply'] === 'theme') {
	cache_delete('', 'wiki-helper-info');
	if(!theme_convert(THEME)) {
	    message_error(l('Could not apply theme preferences.'));
	} else if(isset($args['first']) && $args['first'] === 'true') {
	    header('Location: ' . url_make());
	    exit(0);
	} else {
	    header('Location: ' . url_make('', array('option' => 'admin_setup')));
	    exit(0);
	}
    }

    $form = dom_append_form($dom,array(
				'option' => 'admin_setup',
				'apply' => 'site',
				'first' => default_value($args['first'], '')), 'setup', 'POST');
    dom_set_attribute($form, 'enctype', 'multipart/form-data');

    $categories = array();
    $category_names = array(
	'site' => l('Basic'),
	'theme' => l('Appearance'),
	'debug' => l('Debug'),
	);
    
    foreach($SETUP_CONSTANTS as $key => &$setup_constant) {
	if(!empty($setup_constant['debug']) && DEBUG_MODE !== 'true')
	    continue;

	$category = default_value($setup_constant['category'], 'site');

	if(!isset($categories[$category])) {
	    $fieldset = dom_append_child($form, 'fieldset');
	    $legend = dom_append_child($fieldset, 'legend');
	    dom_append_text($legend, default_value($category_names[$category], $category));
	    $categories[$category] = dom_append_child($fieldset, 'dl');
	}
	$dl = $categories[$category];
	
	$dt = dom_append_child($dl, 'dt');
	dom_append_text($dt, l($setup_constant['description']));

	$dd = dom_append_child($dl, 'dd');
	if(!empty($setup_constant['error']))
	    dom_append_class($dd, 'error');
	
	$type = default_value($setup_constant['type'], 'string');
	if(isset($values[$key]))
	    $value = $values[$key];
	else
	    $value = $setup_constant['default'];
	$name = 'const_' . $key;
	switch($type) {
	case 'select':
	    $options = array_map('l', $setup_constant['options']);
	    dom_append_selector($dd, $name, $options, $value);
	    break;
	case 'timezone':
	    $options = time_get_timezones();
	    ksort($options);
	    dom_append_selector($dd, $name, $options, $value);
	    break;
	case 'theme':
	    $options = theme_get_themes();
	    ksort($options);
	    dom_append_selector($dd, $name, $options, $value);
	    break;
	case 'color':
	    $attributes = array('name' => $name,
				'type' => 'text',
				'value' => $value);
	    if(empty($setup_constant['allowempty']))
		$attributes['required'] = '';
	    
	    dom_append_child($dd, 'input', 'color {hash:true,caps:false,adjust:false}', $attributes);
	    head_tags_add_javascript('resource/script/jscolor/jscolor.js');
	    break;
	case 'theme_file_image':
	    $label = dom_append_child($dd, 'label', 'file_upload');
	    $file = dom_append_child($label, 'input', '',
				     array('type' => 'file', 'name' => $name,
					   'accept' => 'image/*'));
	    if(!empty($setup_constant['allowempty'])) {
		$label = dom_append_child($dd, 'label');
		$check = dom_append_child($label, 'input', '',
					  array(
					      'type' => 'checkbox',
					      'name' => $name . '_delete',
					      'value' => 'on',
					      ));
		dom_append_text($label, l('Delete'));
	    }

	    if($value !== '') {
		dom_append_text($dd, l('Current image') . ' ');
		dom_append_child($dd, 'img', 'current_image', array('src' => $value, 'alt' => l('Current image')));
	    }

	    break;
	case 'string':
	case 'pagename':
	default:
	    $attributes = array('name' => $name,
				'type' => 'text',
				'value' => $value);
	    if(empty($setup_constant['allowempty']))
		$attributes['required'] = '';
	    dom_append_child($dd, 'input', '', $attributes);
	    break;
	}

	if(isset($setup_constant['note'])) {
	    $small = dom_append_child($dd, 'small');
	    dom_append_text($small, l($setup_constant['note']));
	}
    }
    
    $submit = dom_append_child($form, 'input', '', array('type' => 'submit', 'value' => l('Apply')));

    return false;
}

?>
